import uuid

from datetime import date, datetime, timedelta
from typing import Optional

import barker.schemas.guest_book as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.config import settings
from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.customer import Customer
from ..models.guest_book import GuestBook
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.GuestBook)
def save(
    data: schemas.GuestBookIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> schemas.GuestBook:
    try:
        customer: Customer = db.query(Customer).filter(Customer.phone == data.phone).first()
        if customer is None:
            customer = Customer(
                name=data.name,
                phone=data.phone,
                address=data.address,
            )
            db.add(customer)
        else:
            customer.name = data.name or customer.name
            customer.address = data.address or customer.address
        item = GuestBook(pax=data.pax, customer=customer)
        db.add(item)
        db.commit()
        return guest_book_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.GuestBook)
def update(
    id_: uuid.UUID,
    data: schemas.GuestBookIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> schemas.GuestBook:
    try:
        item: GuestBook = db.query(GuestBook).filter(GuestBook.id == id_).first()
        item.customer.name = data.name
        item.customer.phone = data.phone
        item.customer.address = data.address
        item.pax = data.pax
        db.commit()
        return guest_book_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.GuestBookIn)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> schemas.GuestBookIn:
    try:
        item: GuestBook = db.query(GuestBook).filter(GuestBook.id == id_).first()
        db.delete(item)
        db.commit()
        return blank_guest_book_info()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.GuestBookIn)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> schemas.GuestBook:
    return blank_guest_book_info()


@router.get("/list", response_model=schemas.GuestBookList)
def show_list(
    q: Optional[str] = None,
    db: Session = Depends(get_db),
    user: UserToken = Depends(get_user),
) -> schemas.GuestBookList:
    if q is None or q == "":
        q = date.today()
    else:
        q = datetime.strptime(q, "%d-%b-%Y")
    list_ = (
        db.query(GuestBook)
        .filter(
            GuestBook.date >= q + timedelta(minutes=settings.NEW_DAY_OFFSET_MINUTES - settings.TIMEZONE_OFFSET_MINUTES)
        )
        .filter(
            GuestBook.date
            < q
            + timedelta(
                minutes=settings.NEW_DAY_OFFSET_MINUTES - settings.TIMEZONE_OFFSET_MINUTES,
                days=1,
            )
        )
        .order_by(GuestBook.date)
        .all()
    )

    guest_book = []
    for i, item in enumerate(list_):
        guest_book.insert(
            0,
            schemas.GuestBookListItem(
                id=item.id,
                serial=i + 1,
                name=item.customer.name,
                phone=item.customer.phone,
                pax=item.pax,
                date=item.date,
                status=None if item.status is None else item.status.status,
                tableId=None if item.status is None else item.status.food_table.id,
                voucherId=None if item.status is None else item.status.voucher_id,
                tableName=None if item.status is None else item.status.food_table.name,
            ),
        )
    return schemas.GuestBookList(date=q, list=guest_book)


@router.get("/{id_}", response_model=schemas.GuestBook)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> schemas.GuestBook:
    item: GuestBook = db.query(GuestBook).filter(GuestBook.id == id_).first()
    return guest_book_info(item)


def guest_book_info(item: GuestBook) -> schemas.GuestBook:
    return schemas.GuestBook(
        id=item.id,
        name=item.customer.name,
        phone=item.customer.phone,
        pax=item.pax,
        address=item.customer.address,
        date=item.date,
    )


def blank_guest_book_info() -> schemas.GuestBookIn:
    return schemas.GuestBookIn(name="", phone="", pax=0, address="")
