import uuid

from datetime import date
from operator import and_, or_
from typing import List, Optional

import barker.schemas.menu_category as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy import distinct
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.menu_category import MenuCategory
from ..models.product_version import ProductVersion
from ..schemas.user_token import UserToken
from . import effective_date


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.MenuCategory)
def save(
    data: schemas.MenuCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.MenuCategory:
    try:
        item = MenuCategory(
            name=data.name,
            is_active=data.is_active,
            sort_order=0,
        )
        db.add(item)
        db.commit()
        return menu_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.MenuCategory)
def update(
    id_: uuid.UUID,
    data: schemas.MenuCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.MenuCategory:
    try:
        item: MenuCategory = db.query(MenuCategory).filter(MenuCategory.id == id_).first()
        if item.is_fixture:
            raise HTTPException(
                status_code=status.HTTP_423_LOCKED,
                detail=f"{item.name} is a fixture and cannot be edited or deleted.",
            )
        item.name = data.name
        item.is_active = data.is_active
        db.commit()
        return menu_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.MenuCategoryBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.MenuCategoryBlank:
    try:
        item: MenuCategory = db.query(MenuCategory).filter(MenuCategory.id == id_).first()
        if item is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Menu Category not found",
            )
        elif item.is_fixture:
            raise HTTPException(
                status_code=status.HTTP_423_LOCKED,
                detail=f"{item.name} is a fixture and cannot be edited or deleted.",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_501_NOT_IMPLEMENTED,
                detail="Menu Category deletion not implemented",
            )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.MenuCategoryBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.MenuCategoryBlank:
    return menu_category_blank()


@router.get("/list", response_model=List[schemas.MenuCategory])
def show_list(
    p: Optional[bool] = None,
    date_: date = Depends(effective_date),
    db: Session = Depends(get_db),
    user: UserToken = Depends(get_user),
) -> List[schemas.MenuCategory]:

    if p is not None:
        sq = db.query(distinct(ProductVersion.menu_category_id)).filter(
            and_(
                or_(
                    ProductVersion.valid_from == None,  # noqa: E711
                    ProductVersion.valid_from <= date_,
                ),
                or_(
                    ProductVersion.valid_till == None,  # noqa: E711
                    ProductVersion.valid_till >= date_,
                ),
            )
        )
        query = (
            db.query(MenuCategory)
            .filter(MenuCategory.id.in_(sq))
            .order_by(MenuCategory.sort_order)
            .order_by(MenuCategory.name)
            .all()
        )
    else:
        query = db.query(MenuCategory).order_by(MenuCategory.sort_order).order_by(MenuCategory.name).all()
    return [menu_category_info(item) for item in query]


@router.get("/{id_}", response_model=schemas.MenuCategory)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.MenuCategory:
    item: MenuCategory = db.query(MenuCategory).filter(MenuCategory.id == id_).first()
    return menu_category_info(item)


# @view_config(
#     request_method="POST", route_name="v1_menu_categories_list", renderer="json", permission="Products", trans=True,
# )
# def sort_order(request):
#     json = request.json_body
#     for index, item in enumerate(json):
#         request.dbsession.query(MenuCategory).filter(MenuCategory.id == uuid.UUID(item["id"])).update(
#             {MenuCategory.sort_order: index}
#         )
#     return True


def menu_category_info(item: MenuCategory) -> schemas.MenuCategory:
    return schemas.MenuCategory(
        id=item.id,
        name=item.name,
        isActive=item.is_active,
        isFixture=item.is_fixture,
        sortOrder=item.sort_order,
    )


def menu_category_blank() -> schemas.MenuCategoryBlank:
    return schemas.MenuCategoryBlank(
        name="",
        isActive=True,
        isFixture=False,
        sortOrder=0,
    )
