import uuid

from typing import List

import barker.schemas.modifier as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.modifier import Modifier
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Modifier)
def save(
    data: schemas.ModifierIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    try:
        item = Modifier(
            name=data.name,
            show_in_bill=data.show_in_bill,
            price=data.price,
            modifier_category_id=data.modifier_category.id_,
            is_active=data.is_active,
        )
        db.add(item)
        db.commit()
        return modifier_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Modifier)
def update(
    id_: uuid.UUID,
    data: schemas.ModifierIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    try:
        item: Modifier = db.query(Modifier).filter(Modifier.id == id_).first()
        item.name = data.name
        item.show_in_bill = data.show_in_bill
        item.price = data.price
        item.modifier_category_id = data.modifier_category.id_
        item.is_active = data.is_active
        db.commit()
        return modifier_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.ModifierBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.ModifierBlank:
    try:
        item: Modifier = db.query(Modifier).filter(Modifier.id == id_).first()
        db.delete(item)
        db.commit()
        return modifier_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.ModifierBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.ModifierBlank:
    return modifier_blank()


@router.get("/list", response_model=List[schemas.Modifier])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.Modifier]:
    return [modifier_info(item) for item in db.query(Modifier).order_by(Modifier.name).all()]


@router.get("/{id_}", response_model=schemas.Modifier)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    item: Modifier = db.query(Modifier).filter(Modifier.id == id_).first()
    return modifier_info(item)


def modifier_info(item: Modifier) -> schemas.Modifier:
    return schemas.Modifier(
        id=item.id,
        name=item.name,
        showInBill=item.show_in_bill,
        price=item.price,
        isActive=item.is_active,
        modifierCategory=schemas.ModifierCategoryLink(
            id=item.modifier_category.id,
            name=item.modifier_category.name,
        ),
    )


def modifier_blank() -> schemas.ModifierBlank:
    return schemas.ModifierBlank(
        name="",
        showInBill=False,
        price=0,
        isActive=True,
    )
