import uuid

from typing import List

import barker.schemas.sale_category as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.sale_category import SaleCategory
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.SaleCategory)
def save(
    data: schemas.SaleCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.SaleCategory:
    try:
        item = SaleCategory(name=data.name, discount_limit=round(data.discount_limit, 2), tax_id=data.tax.id_)
        db.add(item)
        db.commit()
        return sale_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.SaleCategory)
def update(
    id_: uuid.UUID,
    data: schemas.SaleCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.SaleCategory:
    try:
        item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
        item.name = data.name
        item.discount_limit = round(data.discount_limit, 2)
        item.tax_id = data.tax.id_
        db.commit()
        return sale_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.SaleCategoryBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.SaleCategoryBlank:
    try:
        item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
        if item is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Sale Category not found",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_501_NOT_IMPLEMENTED,
                detail="Sale Category deletion not implemented",
            )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.SaleCategoryBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.SaleCategoryBlank:
    return sale_category_blank()


@router.get("/list", response_model=List[schemas.SaleCategory])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.SaleCategory]:
    return [sale_category_info(item) for item in db.query(SaleCategory).order_by(SaleCategory.name).all()]


@router.get("/for-discount", response_model=List[schemas.SaleCategoryForDiscount])
def for_discount(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.SaleCategory]:
    return [
        schemas.SaleCategoryForDiscount(id=item.id, name=item.name, discountLimit=item.discount_limit, discount=0)
        for item in db.query(SaleCategory).filter(SaleCategory.discount_limit != 0).order_by(SaleCategory.name).all()
    ]


@router.get("/{id_}", response_model=schemas.SaleCategory)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
) -> schemas.SaleCategory:
    item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
    return sale_category_info(item)


def sale_category_info(item: SaleCategory) -> schemas.SaleCategory:
    return schemas.SaleCategory(
        id=item.id,
        name=item.name,
        discountLimit=item.discount_limit,
        tax=schemas.TaxLink(id=item.tax_id, name=item.tax.name, rate=item.tax.rate),
    )


def sale_category_blank() -> schemas.SaleCategoryBlank:
    return schemas.SaleCategoryBlank(name="", discountLimit=1)
